"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checker = undefined;

var _codeGenerator = require("./codeGenerator");

var _codeGenerator2 = _interopRequireDefault(_codeGenerator);

var _Comment = require("./Comment");

var _Comment2 = _interopRequireDefault(_Comment);

var _Options = require("./Options");

var _Options2 = _interopRequireDefault(_Options);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * TypeChecker class generates type checking code based on JSDoc comments in
 * selected environments.
 */
class TypeChecker {
  /**
   * Gets the plugin object itself.
   *
   * @see https://github.com/thejameskyle/babel-handbook/blob/master/translations/en/plugin-handbook.md#toc-writing-your-first-babel-plugin
   * @param {object} babel The current `babel` object.
   * @return {object} The plugin object.
   */
  getPluginObject({
    types: t
  }) {
    if (!t) {
      throw new TypeError('The first argument must be the current babel object.');
    }

    return {
      visitor: {
        ClassMethod: (path, state) => {
          const options = new _Options2.default(state.opts);

          if (!options.shouldGenerateTypeCheckingCode()) {
            return;
          }

          const {
            node: classNode
          } = path.findParent(({
            node
          }) => node && node.type === 'ClassDeclaration');

          const properClassComment = this._findProperComment(classNode, options);

          const {
            node
          } = path;

          const properComment = this._findProperComment(node, options, !properClassComment);

          if (!properComment) {
            return;
          }

          const generatedNodes = _codeGenerator2.default.generateNodes(options.checkingTemplate, properComment);

          if (generatedNodes) {
            node.body.body.unshift(...generatedNodes);
          }
        }
      }
    };
  }

  _findProperComment(node, options, mustHaveCheckerTag = true) {
    const {
      leadingComments
    } = node;

    if (!leadingComments) {
      return null;
    }

    let properComment = null;
    [...leadingComments].reverse().some(commentBlock => {
      if (commentBlock.type === 'CommentBlock') {
        const comment = new _Comment2.default(`/*${commentBlock.value}*/`);

        if (!mustHaveCheckerTag || comment.hasTag(options.checkerTag)) {
          properComment = comment;
          return true;
        }
      }
    });
    return properComment;
  }

}

const checker = exports.checker = new TypeChecker();
exports.default = checker.getPluginObject.bind(checker);